using System;
using System.IO;
using UnityEngine;
using UnityEngine.XR.MagicLeap;

namespace MagicLeap.Examples
{
    public class RecordAudioUI : MonoBehaviour
    {
        private const string TAG = "[RecordAudioUI]";
        private const int AUDIO_CLIP_LENGTH_SECONDS = 3600;
        private const int AUDIO_CLIP_FREQUENCY_HERTZ = 48000;

        private bool isCapturing = false;
        private string deviceMicrophone = string.Empty;
        private MLAudioInput.BufferClip mlAudioBufferClip;


        private bool isRecording = false;
        private string wavFilePath;

        private readonly MLPermissions.Callbacks permissionCallbacks = new MLPermissions.Callbacks();

        //    //Listener for "Start recording" click ("yes" button on previous UI)
        public GameEventListener onRequestNarrativeYesClickedListener;
        //    //Listener for "End Recording" click
        public GameEventListener onEndRecordingClickListener;
        //    // Reference to the GameEvent that should be raised when data is uploaded
        public GameEvent onDataUploadedEvent;

        private DataController dataController;

        private static string fileName;
        private static string startTime;
        private static int startTimeInMillis;
        private static string endTime;
        private static int endTimeInMillis;
        private static string audioDuration;

        private const string validFileFormat = ".wav";


        void Awake()
        {
            permissionCallbacks.OnPermissionGranted += OnPermissionGranted;
            permissionCallbacks.OnPermissionDenied += OnPermissionDenied;
            permissionCallbacks.OnPermissionDeniedAndDontAskAgain += OnPermissionDenied;

            MLPermissions.RequestPermission(MLPermission.RecordAudio, permissionCallbacks);

            dataController = GameObject.FindObjectOfType<DataController>();


            //        //When "Yes" button is click
            onRequestNarrativeYesClickedListener.Response.AddListener(StartRecording);
            //        //When "end recording" is pressed, run the StopRecording method
            onEndRecordingClickListener.Response.AddListener(StopRecording);
        }

        void OnDestroy()
        {
            StopRecording();
            permissionCallbacks.OnPermissionGranted -= OnPermissionGranted;
            permissionCallbacks.OnPermissionDenied -= OnPermissionDenied;
            permissionCallbacks.OnPermissionDeniedAndDontAskAgain -= OnPermissionDenied;
        }

        //public void StartRecording()
        //{
        //    DateTime currentTime = DateTime.Now;

        //    fileName = "audio_" + currentTime.ToString("yyyyMMdd_HHmmss");
        //    startTime = currentTime.ToString("HH:mm:ss");
        //    startTimeInMillis = currentTime.Millisecond;
        //    //wavFilePath = System.IO.Path.Combine(Application.persistentDataPath, fileName);
        //    wavFilePath = System.IO.Path.Combine("/storage/emulated/0/Android/data/com.concepthealth.arconsult/files", fileName);

        //    Debug.LogError(TAG + " Inside StartRecording Function");

        //    if (!MLPermissions.CheckPermission(MLPermission.RecordAudio).IsOk)
        //    {
        //        Debug.LogError(TAG + $"StartRecording() cannot start, {MLPermission.RecordAudio} not granted.");
        //        return;
        //    }

        //    if (isCapturing)
        //    {
        //        Debug.LogWarning(TAG + "Already capturing audio!");
        //        return;
        //    }

        //    isCapturing = true;
        //    mlAudioBufferClip = new MLAudioInput.BufferClip(MLAudioInput.MicCaptureType.VoiceCapture, AUDIO_CLIP_LENGTH_SECONDS, MLAudioInput.GetSampleRate(MLAudioInput.MicCaptureType.VoiceCapture));

        //}


        public void StartRecording()
        {
            try
            {
                Debug.Log(TAG + " Attempting to start recording...");

                if (isCapturing)
                {
                    Debug.LogWarning(TAG + " Recording is already in progress.");
                    return;
                }

                if (!MLPermissions.CheckPermission(MLPermission.RecordAudio).IsOk)
                {
                    Debug.LogError(TAG + $"Cannot start recording as {MLPermission.RecordAudio} has not been granted.");
                    return;
                }

                DateTime currentTime = DateTime.Now;

                if (currentTime == null)
                {
                    Debug.LogError(TAG + " Failed to get the current time.");
                    return;
                }

                fileName = "audio_" + currentTime.ToString("yyyyMMdd_HHmmss");
                startTime = currentTime.ToString("HH:mm:ss");
                startTimeInMillis = currentTime.Millisecond;

                if (string.IsNullOrEmpty(fileName))
                {
                    Debug.LogError(TAG + " Failed to create a filename.");
                    return;
                }

                wavFilePath = System.IO.Path.Combine(Application.persistentDataPath, fileName);

                if (string.IsNullOrEmpty(wavFilePath))
                {
                    Debug.LogError(TAG + " Failed to set the audio file path.");
                    return;
                }

                Debug.Log(TAG + " Preparing audio buffer...");
                mlAudioBufferClip = new MLAudioInput.BufferClip(MLAudioInput.MicCaptureType.VoiceCapture, AUDIO_CLIP_LENGTH_SECONDS, MLAudioInput.GetSampleRate(MLAudioInput.MicCaptureType.VoiceCapture));

                if (mlAudioBufferClip == null)
                {
                    Debug.LogError(TAG + " Failed to initialize audio buffer.");
                    return;
                }

                isCapturing = true;
                Debug.Log(TAG + " Recording has started successfully.");
            }
            catch (Exception ex)
            {
                Debug.LogError(TAG + " An error occurred while attempting to start recording: " + ex.Message);
            }
        }

        public void StopRecording()
        {
            Debug.Log(TAG + " Inside StopRecording Function");
            if (!isCapturing)
            {
                return;
            }

            mlAudioBufferClip?.Dispose();

            // Save the audio data to a file before disposing the buffer and uploading.
            SaveToFile(wavFilePath, AudioFileType.WAV); // Added this line to save the audio data.


            mlAudioBufferClip = null;

            isCapturing = false;


            DateTime currentTime = DateTime.Now;

            endTime = currentTime.ToString("HH:mm:ss");
            endTimeInMillis = currentTime.Millisecond;

            audioDuration = (endTimeInMillis - startTimeInMillis).ToString();

            int appointment_id = PlayerPrefs.GetInt("appointment_id");
            startTime = (DateTime.Now).ToString();

            ConsultationData consultationData = dataController.CreateConsultationDataWithNoConsent(
                appointment_id,
                startTime.ToString(),
                endTime.ToString(),
                dataController.GetLocalIPAddress(),
                "completed",
                wavFilePath,
                "wav",
                "???",
                audioDuration);
            dataController.UploadConsultationData(consultationData, onDataUploadedEvent);
            UpdateAppointmentStatus(appointment_id);
        }

        private void UpdateAppointmentStatus(int id)
        {
            Appointment currentAppointment = dataController.GetAppointmentByID(id);
            AppointmentUpdate appointmentUpdate = new AppointmentUpdate();
            appointmentUpdate.status = "completed";
            appointmentUpdate.start_at = currentAppointment.attributes.started_at;
            appointmentUpdate.ended_at = currentAppointment.attributes.ended_at;
            dataController.UpdateAppointment(appointmentUpdate, currentAppointment.id);
        }

        public void SaveToFile(string path, AudioFileType type)
        {
            if (mlAudioBufferClip == null)
            {
                Debug.LogError(TAG + " No audio data available to save!");
                return;
            }

            var audioClip = mlAudioBufferClip.FlushToClip();

            switch (type)
            {
                case AudioFileType.WAV:
                    SaveToWav(path, audioClip);
                    break;
                default:
                    Debug.LogError(TAG + " Invalid or unsupported file type specified!");
                    break;
            }
        }

        private void SaveToWav(string filename, AudioClip clip)
        {
            var filepath = Path.Combine(Application.persistentDataPath, filename + ".wav");

            byte[] audioBytes = WavUtility.FromAudioClip(clip);
            File.WriteAllBytes(filepath, audioBytes);

            Debug.Log(TAG + $" Saved to {filepath}");
        }

        private void OnPermissionDenied(string permission)
        {
            Debug.LogError(TAG + $" Failed to get requested permission {permission}, disabling script.");
            enabled = false;
            return;
        }

        private void OnPermissionGranted(string permission)
        {
            Debug.Log(TAG + $" Succeeded in requesting {permission}.");
        }
    }

    public enum AudioFileType
    {
        WAV
    }

    // This is a basic WAV utility for AudioClip
    public static class WavUtility
    {
        const uint HEADER_SIZE = 44;

        public static byte[] FromAudioClip(AudioClip clip)
        {
            MemoryStream stream = new MemoryStream();
            using (BinaryWriter writer = new BinaryWriter(stream))
            {
                uint totalSampleCount = (uint)(clip.samples * clip.channels);
                uint byteCount = totalSampleCount * sizeof(short);
                uint fileSize = HEADER_SIZE + byteCount;

                writer.Write("RIFF".ToCharArray());
                writer.Write(fileSize);
                writer.Write("WAVE".ToCharArray());
                writer.Write("fmt ".ToCharArray());
                writer.Write(16);
                writer.Write((ushort)1);
                writer.Write((ushort)clip.channels);
                writer.Write(clip.frequency);
                writer.Write(clip.frequency * 2);
                writer.Write((ushort)2);
                writer.Write((ushort)16);
                writer.Write("data".ToCharArray());
                writer.Write(byteCount);

                float[] samples = new float[clip.samples];
                clip.GetData(samples, 0);

                Int16[] intData = new Int16[samples.Length];
                byte[] bytesData = new byte[samples.Length * 2];

                int rescaleFactor = 32767; //to convert float to Int16

                for (int i = 0; i < samples.Length; i++)
                {
                    intData[i] = (short)(samples[i] * rescaleFactor);
                    byte[] byteArr = BitConverter.GetBytes(intData[i]);
                    byteArr.CopyTo(bytesData, i * 2);
                }

                writer.Write(bytesData);
            }

            return stream.ToArray();
        }
    }
}
